package net.sf.itcb.addons.monitoring.track;

import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;

import net.sf.itcb.addons.monitoring.track.appender.ItcbAppender;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbConstants;

/**
 * This class gets the transaction id in the current thread or generates it if it doesn't exist<br/>
 * In a transaction, many track beans can be stored at each monitoring managed layer. Those beans are stored in ThreadLocal.<br/>
 * When the first opened bean is close (end of the process in this method), transaction is closed and data are stored thanks to ItcbAppender.
 * For those who wants to trace transactionId in logs, it is stored with MDC mecanism. associated to the key : <code>itcbTransactionId</code>
 * 
 * @author Pierre Le Roux
 *
 */
public class MonitoringTransactionManager {
	
	/**
	 * Transaction ID in log files using <code>%X{itcbTransactionId}</code> in log4j.xml patterns (for example).
	 */
	public static final String MDC_TRANSACTION_ID = "itcbTransactionId";
	
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	private ItcbAppender itcbAppender;
		
	@Required
	public void setItcbAppender(ItcbAppender itcbAppender) {
		this.itcbAppender = itcbAppender;
	}
	 
	 /**
	  * the transactionId management
	  */
	 private ThreadLocal < ItcbTrackBean > transaction = new ThreadLocal < ItcbTrackBean > ();	 

	 private ThreadLocal < Integer > rootPhaseId = new ThreadLocal < Integer > () {
		
		 @Override
		 protected Integer initialValue() {
			 return 0;
		 }
		 
		 @Override
		 public Integer get() {
			 int value = super.get() + 1;
			 set(value);
			 return value;
		 }		 
	 };
	 
	 
	/**
	  * Set the transaction Id in ThreadLocal : 
	  * usefull when we received a message from another service with an already created transactionId
	  * @param transactionId
	  */
	 public void setCurrentTrackBean(ItcbTrackBean itcbTrackBean) {
		 ItcbTrackBean parent = transaction.get();
		 if(parent == null) {
			 if(itcbTrackBean.getTransactionId() == null) {
				 itcbTrackBean.setTransactionId(UUID.randomUUID().toString());
			 }
			 itcbTrackBean.setPhaseId(rootPhaseId.get());
			 itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
			 MDC.put(MDC_TRANSACTION_ID, itcbTrackBean.getTransactionId()); 
		 }
		 else {
			 itcbTrackBean.setTransactionId(parent.getTransactionId());
			 itcbTrackBean.setParent(parent);
			 itcbTrackBean.setParentPhaseId(parent.getCompletePhaseId());
			 itcbTrackBean.setPhaseId(parent.getChildren().size() + 1);
			 parent.getChildren().add(itcbTrackBean);
		 }
		 transaction.set(itcbTrackBean);
	 }
	 
	 /**
	  * get the transactionId in threadlocal
	  * @return the transaction
	  */
	 public ItcbTrackBean getCurrentTrackBean() {
		 return transaction.get();
	 }

	
	 /**
	  * End the current action and close the transaction if it was the first action
	  * @param itcbTrackBean
	  */
  	 public void endCurrentTrackBean() {
  		ItcbTrackBean itcbTrackBean = getCurrentTrackBean();  
		transaction.set(itcbTrackBean.getParent());
		
		if(itcbTrackBean.getParent() == null) {
			itcbAppender.loopTrack(itcbTrackBean);
			transaction.remove();
			rootPhaseId.remove();
			MDC.remove(MDC_TRANSACTION_ID);
		}
	 }
}

